@@ -1378,6 +1378,12 @@ do_query_info (GVfsBackend *backend,
 
     LIBMTP_devicestorage_t *storage;
     gboolean found = FALSE;
+
+    int ret = LIBMTP_Get_Storage (device, LIBMTP_STORAGE_SORTBY_NOTSORTED);
+    if (ret != 0) {
+      fail_job (G_VFS_JOB (job), device);
+      goto exit;
+    }
     for (storage = device->storage; storage != 0; storage = storage->next) {
       if (storage->id == entry->storage) {
         DEBUG ("(I) found storage %X\n", storage->id);
@@ -1459,6 +1465,12 @@ do_query_fs_info (GVfsBackend *backend,
     }
 
     LIBMTP_devicestorage_t *storage;
+
+    int ret = LIBMTP_Get_Storage (device, LIBMTP_STORAGE_SORTBY_NOTSORTED);
+    if (ret != 0) {
+      fail_job (G_VFS_JOB (job), device);
+      goto exit;
+    }
     for (storage = device->storage; storage != 0; storage = storage->next) {
       if (storage->id == entry->storage) {
         get_storage_info (storage, info);
@@ -1575,6 +1587,7 @@ do_pull (GVfsBackend *backend,
   GFile *local_file = NULL;
   GFileInfo *local_info = NULL;
   GFileInfo *info = NULL;
+  guint64 mtime;
 
   CacheEntry *entry = get_cache_entry (G_VFS_BACKEND_MTP (backend), source);
   if (entry == NULL) {
@@ -1681,6 +1694,19 @@ do_pull (GVfsBackend *backend,
     fail_job (G_VFS_JOB (job), device);
     goto exit;
   }
+
+  /* Ignore errors here. Failure to copy metadata is not a hard error */
+  mtime = g_file_info_get_attribute_uint64 (info,
+                                            G_FILE_ATTRIBUTE_TIME_MODIFIED);
+  g_file_set_attribute_uint64 (local_file,
+                               G_FILE_ATTRIBUTE_TIME_MODIFIED, mtime,
+                               G_FILE_QUERY_INFO_NONE,
+                               G_VFS_JOB (job)->cancellable, NULL);
+  g_file_set_attribute_uint32 (local_file,
+                               G_FILE_ATTRIBUTE_TIME_MODIFIED_USEC, 0,
+                               G_FILE_QUERY_INFO_NONE,
+                               G_VFS_JOB (job)->cancellable, NULL);
+
   /* Attempt to delete object if requested but don't fail it it fails. */
   if (remove_source) {
     DEBUG ("(I) Removing source.\n");
